**The idea:** we agree on the overall roadmap as an enumeration of things that
need to be done on a high level. Each item in the list then gets a GitLab
milestone, unless there is a good argument to not have a milestone for a
particular item.

Once we have a list, we sync on who's the _main driver_ for each item in it.

The main driver plans their milestone in sufficient detail to communicate **what
will be ready once it's done**, and **who will be involved in the work**.

If anyone else than the main driver needs to be involved, it's up to the main
driver to ensure that these people know what to do (e.g., express this in the
milestone and below where milestones are "claimed" to ensure we get a good
overview of this).

Other than this there's no one-fit-all for how to define/plan a milestone. Feel
free to either write a wall in the milestone description, create a large number
of small issues, or something in between with, say, larger placeholder issues.

Ask someone to do a review of the planned milestone for you.

We hope this will help us get organized internally, but also to communicate our
roadmap better to those that are external to the project and peeking in.

Once we're happy with this document, rgdd will put it up for decision on a
weekly. Reach out to rgdd if you have any questions or comments about this.

# 2023-11-06

This document provides a sketch of the current roadmap and involved people.

## Summary of changes

The outlined roadmap ultimately aims to produce a getting-started tutorial that
shows how to stboot a system with secure boot enabled (custom keys),
network-fetching of UKI-formatted OS packages, and Sigsum as well as remote
attestation for verification. UEFI support only. See more details below.

There's no recent roadmap to compare against, next time maybe.

The roadmap will be renewed on 2023-12-18, unless we conclude that it is not
helpful to create this type of document for planning purposes. Or if someone has
a better idea of how to do this, it can be considered until then.

## Roadmap

In some kind of logical order, although not a strict priority list as the
project involves many people that are sometimes working on orthogonal things:

1. Address feature requests from early adopters that alter no reference specs
   - https://git.glasklar.is/groups/system-transparency/-/milestones/8#tab-issues
2. Wrap-up the first take on docs.system-transparency.org
   - https://git.glasklar.is/groups/system-transparency/-/milestones/11#tab-issues
3. Make status-quo releases of ST components
   - https://git.glasklar.is/groups/system-transparency/-/milestones/14#tab-issues
4. Update www.system-transparency.org and mailing lists
   - https://git.glasklar.is/groups/system-transparency/-/milestones/9#tab-issues
5. Improve and simplify CI pipelines
   - https://git.glasklar.is/groups/system-transparency/-/milestones/13#tab-issues
   - comment rgdd: leave core/system-transaprency as is, the "replace with bash
     script" should imo be about moving things from the task stuff into
     stprov/stboot/stmgr CI pipelines where we need it to run. And then we do it
     with a bash script, so it is easy to understand and hack-patch much more
     quickly. (And future of core/system-transparency: probably uncertain, new
     "entry point" is likely docs.sto as discussed with e.g. jens and zaolin.)
5. Adopt UKI and mkosi in OS package tooling
   - https://git.glasklar.is/groups/system-transparency/-/milestones/15
   - comment jens: this should also cover the adaption in stboot to consume OS
     packages in UKI format
7. Finalize / test loading stboot from UEFI (no network yet)
   - https://git.glasklar.is/groups/system-transparency/-/milestones/18#tab-issues
8. Add experimental remote attestation support
   - https://git.glasklar.is/groups/system-transparency/-/milestones/12#tab-issues
9. Add support to Sigsum-verify OS packages
   - https://git.glasklar.is/groups/system-transparency/-/milestones/19#tab-issues
10. Getting started guide that puts all of the above together
    - https://git.glasklar.is/groups/system-transparency/-/milestones/16
11. Automated QA systems to test ST in production on real hardware
    - Requires HPE hardware or supermicro as fallback
    - https://git.glasklar.is/groups/system-transparency/-/milestones/17
12. Competitor Intelligence
    - https://git.glasklar.is/groups/system-transparency/-/milestones/10#tab-issues

Please note that the above is not an exhaustive list of everything we will be
doing, or interpret it as anything not on the list is out-of-scope. View it as
things we have high confidence to work on until _at least_ December 18. In other
words, we are not promising that all of the above will be done until then.

Backlog tasks and clean-up is in-scope on a need-basis. Similarly, support and
bug-fixing is also in-scope and planned separately on a case-by-case basis.

Looking ahead, we will likely pick-and-choose from the following:

- stam
- stvmm
- Testing and quality assurance, includes things like having continuous testing
  on hardware which is (thanks to a testbed) known to work with ST
- Supply chain security, things like how do we track Go dependencies etc
- Alternatives to secure boot custom keys
- Get coreboot supported again
- ...

Please note that this is a non-exhaustive list. It is also subject to change.
You may find more future ideas in "Draft: " milestones or repository issues.

## People

Based on who's regularly involved on IRC, Matrix, and weekly meets:

- doctor-love
  - Provides feedback on a best-effort basis
- fredrik
  - Weekly chair
  - Main responsible for vision, heavily involved in theory and strategy
- jens
  - Tries to help with process things
  - Input and review all over the place this time
  - Drives 7
  - Drives 12
- zaolin
  - Main responsible for useful documentation and themeing
  - Drives 2, will need input and review from jens, rgdd and TBD
  - Drives 6, will need input and review from jens
  - Drives 10, will need help and review from most of you
  - Drives 11, will need help and review from ln5
- kai
  - Drives 5
  - Drives 8
- ln5
  - Point of contact for those who want to deploy ST, knows about things like
    pain points and wish lists of our early adopters
- nisse
  - Drives 1, will need input/review from ln5
  - Drives 9, will need input/review from rgdd
- rgdd
  - Weekly secretary
  - Tries to help with process things
  - Drives 3, will need help from jens and philipp
  - Drives 4, will need help and review from ln5 and zaolin
- quite
  - Works on bootstrapping a new infrastructure on-top of ST

## Milestones

If you're the driver of something, you're responsible to create a milestone and
self-organize around this task with others that need to be involved. When we
update the roadmap the next time, you will either close the milestone (it is
done) or consider what should happen with the things that were backlogged.

Add a link to your milestone in the above roadmap list (and replace the drafty
comments that just helps us know what we're talking about until this happened).
